import java.io.*;
import java.net.*;


/**
 *
 * The class handles UDP network traffic from/to the rcssserver
 *
 *  @author Henrik Sander
 *  @author Michael Helisch
 *  @author Manuel Link
 *  @author Max Urban
 *  @author Cornelius Siller
 *.
 */

public class CommunicationChannel extends Thread
{
    /** socket for incoming, outgoing messages */
    private DatagramSocket socket;
    /** IP address of the server */
    InetAddress serverAddress;
    /** port number of the server */
    private int serverPort;

/**
*	Constructor
*/
    public CommunicationChannel(String serverName, int serverPort, PipedWriter p) {
        try {
            // construct datagram socket and
            // bind it to any available port on the local machine
            this.socket        = new DatagramSocket();
            // set target address
            this.serverAddress = InetAddress.getByName(serverName);
            // set target port
            this.serverPort    = serverPort;
        } catch (SocketException se) {
            System.err.println("CommunicationChannel: Could not create datagram socket.");
            System.exit(1);
        } catch (UnknownHostException uhe) {
            System.err.println("CommunicationChannel: Could not determine the server's IP address.");
            System.exit(1);
        }
	pout = new BufferedWriter(p);
	setPriority(MIN_PRIORITY);
    }


/**	
 *	Receives the messages the server sends
 *
 */
    public String receive() {
        // allocate buffer for server message
        byte[] buffer = new byte[2048];
        // create datagram packet for receiving
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
        try {
            // receive packet, the method blocks until a datagram arrives
            socket.receive(packet);
        } catch (IOException ioe) {
            System.err.println("CommmunicationChannel: Could not receive datagram packet.");
        }
        // get the server port for the following actions
        serverPort = packet.getPort();
        // return the message as string
        String r = new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  // cut off null characters & \n in buffer
//	System.out.println("received: "+r);
	return r;

    } // receive()

/**
 * 	Sends the messages the actor class creates
 *
 */ 
    public void send(String message) {
        try {
            // convert the message to a byte array
//	System.out.println("call to send(): serverport = "+serverPort + " message = "+message);
	    byte[] buffer = message.getBytes();
            // create the packet from the buffer and set the server's IP address and port
            DatagramPacket packet =
                new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
            // send the packet
            socket.send(packet);
	    comm_setup = true;
        } catch (IOException ioe) {
            System.err.println("CommunicationChannel: Could not send datagram packet.");
        }
    } // send()

/**
 *
 *	Constantly receives messages and makes them available for the
 *	actor class for further use
 *
 */
  
	public void run()
	{
	while (true)
		{
		if (comm_setup)
			{
//			System.out.println("call to receive(), port: "+serverPort);
			String s = receive();
//			System.out.println("receive: "+s);
			try {
				pout.write(s+"\n", 0, s.length()+1);
				pout.flush();
			} catch (IOException ioe) { }
//			try {	//System.out.println("try to sleep 100ms");
//				sleep(100); } catch (InterruptedException ie) {}
			}
		}
	} // run()

	BufferedWriter pout;
	BufferedReader pin;
	boolean comm_setup = false;

} // class

