import java.util.*;

 /**
 *	This class contains the main a.i. of our players. Special functions are to be
 *	found in the classes:
 *	Goalie.java
 *	Defense.java
 *	Midfield.java
 *	Offense.java
 *
 *
 *  @author Henrik Sander
 *  @author Michael Helisch
 *  @author Manuel Link
 *  @author Max Urban
 *  @author Cornelius Siller 
*/
 
public abstract class Player extends Thread
{

	protected World world;
	protected CommunicationChannel communication;
	
    protected String playOnSide;
	protected String oppGoal; 
    protected int ownNumber;
	protected String playMode;
	double[][] area = new double[11][6];
	double[][] darea = new double[11][6];
	
	boolean initReceived = false;
	
	
	
	
/** 
 * 	the actor module of the client 
 */
    protected Actor  actor;

/** 
 *	the name of the team (for initialization) 
 */
    protected String teamName;
  
/**
 *	this Variable indicates the active player for visual output
 */
	protected boolean active = false;
	protected Team team;
	
	boolean defenseStrategy;
	
    private boolean offside = false; // used in status decision (play())
    private int turnCounter = 1; // used by method goHome()
  
  
/** 
 *	superstati 
 */
    final protected int START         = 0;
    final protected int PLAYING       = 1;
    final protected int WAITING 	  = 2;
	
    protected int status = START;
	
	
/** 
 *	stati for PLAYING 
 */
	final protected int INHOMEZONE = 0;
	final protected int NOTINHOMEZONE = 1;
	protected int homeZoneStatus;
	
	
/** 
 *	stati for INHOMEZONE 
 */
	final protected int INHOMEPOSITION = 0;
	final protected int APPROACHBALL = 1;
	final protected int RETURNTOHOMEPOSITION = 2;
	protected int homePositionStatus =  RETURNTOHOMEPOSITION;
	


/** 
 *	the actual timestamp of last visual event received 
 */
    protected int time = -1;
	 
	 
/**
*	The constructor of the Player class
*	contains the information about the: 
*		starting position
*		home position
*		home zone 
*	of every initialized player according to his type and number
*/	 
	public Player()
	{
	
		// Defensive Playing
		// Goalie InitField and Position		
		darea[0][0] = -50.0; // x coordinate HomePosition
		darea[0][1] =   0.0; // y coordinate HomePosition
		darea[0][2] = -18.0; // top border
		darea[0][3] = -37.0; // right border
		darea[0][4] =  18.0; // bottom border
		darea[0][5] = -52.8; // left border

		// Defence 1 InitField and Position		
		darea[1][0] = -34.9; // x coordinate HomePosition
		darea[1][1] = -20.0; // y coordinate HomePosition
		darea[1][2] = -36.0; // top border
		darea[1][3] = -17.0; // right border
		darea[1][4] = -11.0; // bottom border
		darea[1][5] = -52.8; // left border	 
			 

		// Defence 2 InitField and Position		
		darea[2][0] = -34.9; // x coordinate HomePosition
		darea[2][1] =   0.0; // y coordinate HomePosition
		darea[2][2] = -12.0; // top border
		darea[2][3] = -23.0; // right border
		darea[2][4] =  12.0; // bottom border
		darea[2][5] = -48.0; // left border	 

		// Defence 3 InitField and Position		
		darea[3][0] = -34.9; // x coordinate HomePosition
		darea[3][1] =  20.0; // y coordinate HomePosition
		darea[3][2] =  11.0; // top border
		darea[3][3] = -17.0; // right border
		darea[3][4] =  36.0; // bottom border
		darea[3][5] = -52.8; // left border	 

		// Midfield 4 InitField and Position		
		darea[4][0] = -15.0; // x coordinate HomePosition
		darea[4][1] = -23.0; // y coordinate HomePosition
		darea[4][2] = -36.0; // top border
		darea[4][3] =  12.0; // right border
		darea[4][4] = -12.0; // bottom border
		darea[4][5] = -30.0; // left border

		// Midfield 5 InitField and Position		
		darea[5][0] = -18.0; // x coordinate HomePosition
		darea[5][1] =  -6.5; // y coordinate HomePosition
		darea[5][2] = -20.0; // top border
		darea[5][3] =  17.5; // right border
		darea[5][4] =   7.0; // bottom border
		darea[5][5] = -26.0; // left border	 

		// Midfield 6 InitField and Position		
		darea[6][0] =  -18.0; // x coordinate HomePosition
		darea[6][1] =   6.5; // y coordinate HomePosition
		darea[6][2] =  -7.0; // top border
		darea[6][3] =  17.5; // right border
		darea[6][4] =  20.0; // bottom border
		darea[6][5] = -26.0; // left border	 

		// Midfield 7 InitField and Position		
		darea[7][0] = -15.0; // x coordinate HomePosition
		darea[7][1] =  23.0; // y coordinate HomePosition
		darea[7][2] =  12.0; // top border
		darea[7][3] =  12.0; // right border
		darea[7][4] =  36.0; // bottom border
		darea[7][5] = -30.0; // left border	 	 
		
		// Offence 8 InitField and Position		
		darea[8][0] =  -3.0; // x coordinate HomePosition
		darea[8][1] = -18.0; // y coordinate HomePosition
		darea[8][2] = -36.0; // top border
		darea[8][3] =  52.8; // right border
		darea[8][4] =   5.0; // bottom border
		darea[8][5] =  -3.0; // left border	 	 
		
		// Offence 9 InitField and Position		
		darea[9][0] =  -3.0; // x coordinate HomePosition
		darea[9][1] =  18.0; // y coordinate HomePosition
		darea[9][2] =  -5.0; // top border
		darea[9][3] =  52.8; // right border
		darea[9][4] =  36.0; // bottom border
		darea[9][5] =  -3.0; // left border
		
		// Offence 10 InitField and Position		
		darea[10][0] =  -5.0; // x coordinate HomePosition
		darea[10][1] =   0.0; // y coordinate HomePosition
		darea[10][2] = -15.0; // top border
		darea[10][3] =  52.8; // right border
		darea[10][4] =  15.0; // bottom border
		darea[10][5] = -13.0; // left border	 
		
		
		// Offensive Playing
		// Goalie InitField and Position		
		area[0][0] = -50.0; // x coordinate HomePosition
		area[0][1] =   0.0; // y coordinate HomePosition
		area[0][2] = -18.0; // top border
		area[0][3] = -37.0; // right border
		area[0][4] =  18.0; // bottom border
		area[0][5] = -52.8; // left border

		// Defence 1 InitField and Position		
		area[1][0] = -34.9; // x coordinate HomePosition
		area[1][1] = -20.0; // y coordinate HomePosition
		area[1][2] = -36.0; // top border
		area[1][3] = -17.0; // right border
		area[1][4] = -11.0; // bottom border
		area[1][5] = -52.8; // left border	 
			 

		// Defence 2 InitField and Position		
		area[2][0] = -34.9; // x coordinate HomePosition
		area[2][1] =   0.0; // y coordinate HomePosition
		area[2][2] = -12.0; // top border
		area[2][3] = -23.0; // right border
		area[2][4] =  12.0; // bottom border
		area[2][5] = -48.0; // left border	 

		// Defence 3 InitField and Position		
		area[3][0] = -34.9; // x coordinate HomePosition
		area[3][1] =  20.0; // y coordinate HomePosition
		area[3][2] =  11.0; // top border
		area[3][3] = -17.0; // right border
		area[3][4] =  36.0; // bottom border
		area[3][5] = -52.8; // left border	 

		// Midfield 4 InitField and Position		
		area[4][0] = -5.0; // x coordinate HomePosition
		area[4][1] = -23.0; // y coordinate HomePosition
		area[4][2] = -36.0; // top border
		area[4][3] =  12.0; // right border
		area[4][4] = -12.0; // bottom border
		area[4][5] = -30.0; // left border

		// Midfield 5 InitField and Position		
		area[5][0] =  -8.0; // x coordinate HomePosition
		area[5][1] =  -6.5; // y coordinate HomePosition
		area[5][2] = -20.0; // top border
		area[5][3] =  17.5; // right border
		area[5][4] =   7.0; // bottom border
		area[5][5] = -26.0; // left border	 

		// Midfield 6 InitField and Position		
		area[6][0] =  -8.0; // x coordinate HomePosition
		area[6][1] =   6.5; // y coordinate HomePosition
		area[6][2] =  -7.0; // top border
		area[6][3] =  17.5; // right border
		area[6][4] =  20.0; // bottom border
		area[6][5] = -26.0; // left border	 

		// Midfield 7 InitField and Position		
		area[7][0] = -5.0; // x coordinate HomePosition
		area[7][1] =  23.0; // y coordinate HomePosition
		area[7][2] =  12.0; // top border
		area[7][3] =  12.0; // right border
		area[7][4] =  36.0; // bottom border
		area[7][5] = -30.0; // left border	 	 
		
		// Offence 8 InitField and Position		
		area[8][0] =  25.0; // x coordinate HomePosition
		area[8][1] = -18.0; // y coordinate HomePosition
		area[8][2] = -36.0; // top border
		area[8][3] =  52.8; // right border
		area[8][4] =   5.0; // bottom border
		area[8][5] =   8.0; // left border	 	 
		
		// Offence 9 InitField and Position		
		area[9][0] =  25.0; // x coordinate HomePosition
		area[9][1] =  18.0; // y coordinate HomePosition
		area[9][2] =  -5.0; // top border
		area[9][3] =  52.8; // right border
		area[9][4] =  36.0; // bottom border
		area[9][5] =   8.0; // left border
		
		// Offence 10 InitField and Position		
		area[10][0] =  30.0; // x coordinate HomePosition
		area[10][1] =   0.0; // y coordinate HomePosition
		area[10][2] = -15.0; // top border
		area[10][3] =  52.8; // right border
		area[10][4] =  15.0; // bottom border
		area[10][5] =   3.0; // left border	 
		
	}
	
	
	public boolean isGoalie() {return false;}
	public boolean isDefense() {return false;}
	public boolean isMidfield() {return false;}
	public boolean isOffense() {return false;}
     
	 
	 
/**
*	This method decides, which playstatus is the actual playstatus and starts the
*	special methods for this playstatus.
*	It is started by the run() method in the classes Goalie, Defense, Midfield, Offense
*/	
	
	public void play()
	{
		switch (status)
		{
			case START:
				setStartPosition();
				if (offside)
				{
					status = WAITING;
					offside = false;
				}
				else
				{
					status = WAITING;
					homePositionStatus =  RETURNTOHOMEPOSITION;
				}
				break;
			case PLAYING:
				if(isGoalie())
//					System.out.println("homezonestatus: "+homeZoneStatus+"  homeposstatus: "+homePositionStatus);

				homeZoneStatus = checkHZone(); // Checks if player is in HomeZone
		
				switch (homeZoneStatus)
				{				
					case INHOMEZONE:
//						System.out.println(homePositionStatus);
						switch (homePositionStatus)
						{
							case INHOMEPOSITION:
								waitForBall();
								break;
							case APPROACHBALL:
								runToBall();
								break;
							case RETURNTOHOMEPOSITION:
								goHome();
								break;						
						}
						break; // INHOMEZONE
					case NOTINHOMEZONE:
//						System.out.println("not in homezone " + ownNumber);
						goHome();
						break; // NOTINHOMEZONE						
				} // switch (homeZoneStatus)
				break; // PLAYING
			case WAITING:
				turnToBall(); 
				// and then just do nothing...
				break; // WAITING
		} //switch (status)
	} // play()
	
	
/**
*	method started by status INHOMEPOSITION, can be overwritten if necessary
*/
	public void waitForBall()
	{
		if (!world.myActArea.checkHomePosition()) 
		{
//			System.out.println("inHomepos"+ownNumber);
			homePositionStatus = RETURNTOHOMEPOSITION;
		}
	
		turnCounter = 1;
//		System.out.println("wait for ball " + ownNumber);
		if (!(world.verifyBall())) // already found the ball?
		{
			// no, look around
			actor.turn(40);
		}
		else
		{
			// we found the ball
			
			// what is my position?
			double [] position = world.myself.getTestPosition();
			
			// turn to it if necessary but only if new ball info is available
			if (Math.abs(world.theBall.getNewDirection()) > 1.0 && world.theBall.time()!=time) 
			{
				actor.turn(world.theBall.getNewDirection());
	    		// remember the time of the info the action was based on
				
	    		time=world.theBall.time();
			}
			else if (world.theBall.getNewDistance() > 0.7) 
			{
//				if (ownNumber == 9)
//				{
//					System.out.println(world.myActArea.checkBallInHomeZone());							
//				}
				if (world.myActArea.checkBallInHomeZone())
				{
//					if (ownNumber == 9)
//					{
//						System.out.println(	"will ball!");							
//					}
				
					homePositionStatus = APPROACHBALL;									
				}
			
		
			} 
			else // ball is near
			{
				ballIsNearWhatToDo();		    	
			}
		}
	} // waitForBall()
	
	
/**
*	method started by status APPROACHBALL, can be overwritten if necessary
*/
	public void runToBall()
	{
//		if (ownNumber == 9)
//		{
//			System.out.println("run to Ball");
//		}
		if (!(world.verifyBall())) // already found the ball?
		{
			// no, look around
			actor.turn(40);
		}
		else
		{
			// we found the ball
			
			// what is my position?
//			double [] position = world.myself.getPosition();
			
			// turn to it if necessary but only if new ball info is available
			if (world.theBall.distance() < 1) // ball is near
			{
				ballIsNearWhatToDo();		    	
			}
			
			else if (Math.abs(world.theBall.direction()) > 5.0 && world.theBall.time()!=time) 
			{
				if (world.theBall.futurePos)
				{
					actor.turn(world.theBall.getNewDirection());
					
				}
				else
				{
					actor.turn(world.theBall.direction());
				}
	    		// remember the time of the info the action was based on
				// System.out.println("Turning towards ball");
	    		time=world.theBall.time();
			}
			else // if (world.theBall.distance() > 0.7) 
			{								
				if (world.theBall.getNewDistance() < 2 )
				{
//					if (world.theBall.futurePos)
//					{
//						if (ownNumber == 9)
//						{
//							System.out.println("newDistance : " +world.theBall.getNewDistance());
//						}
						actor.dash(40); // 15 * world.theBall.getNewDistance());
						
//					}
//					else
//					{
//						actor.dash(40); // 15 * world.theBall.distance());
//					}
					 // get slower as we approach the ball
				}
				else if (world.theBall.getNewDistance() < 4 )
				{
					actor.dash(60);
				
				}
				else // if (world.theBall.distance() < 20)
				{
					if (world.myActArea.checkBallInHomeZone())
					{	
						actor.dash(100);
					}
					else
					{
						homePositionStatus = RETURNTOHOMEPOSITION;
					}
					
				}
			
			
		
			} 
		}
	} // runToBall()


/**
*	method started by status RETURNTOHOMEPOSITION, can be overwritten if necessary
*/
	public void goHome()
	{
		homePositionStatus = RETURNTOHOMEPOSITION;
		if (world.myActArea.checkHomePosition()) 
		{
//			System.out.println("inHomepos"+ownNumber);
			homePositionStatus = INHOMEPOSITION;
		}
		if ((world.verifyBall()) && world.myActArea.checkBallInHomeZone()) // already found the ball?
		{
			// no, look around
			homePositionStatus = APPROACHBALL;
		}
		else
		{
//			System.out.println("goHome"+ownNumber);
			if (turnCounter == 1)
			{
				actor.turn(world.myActArea.getHomePositionDirection()); // Wo soll diese Funktion stehen?
				turnCounter = 0;
//				try
//    			{
//    				sleep(100);
//    			}
//    			catch (InterruptedException ire)
//    			{
//    				System.err.println("sleep error");
//    			}	
			}
			else
			{
		
				if (world.myActArea.getHomePositionDistance() < 2)
				{
					actor.dash(10 * world.myActArea.getHomePositionDistance()); // Sollte kein Problem sein
				}
				else
				{
					actor.dash(100); // Sollte kein Problem sein
				}
				turnCounter += 1;
			}
		}
		
	
		
	
	}
	
/**
*	decides what to do with the ball, can be overwritten if necessary
*/
	public void ballIsNearWhatToDo()
	{
		actor.kick(100, world.theGoal.direction());
	} // ballIsNearWhatToDo()
	
/**
*	sets players startPosition, overwritten by all extending classes
*/
	public void setStartPosition()
	{
	}
/**		
*	sets the goal to play on
*/
	public void setPlaySide(String side)
	{
	
		playOnSide = side;
		if (side.equals("r"))
		{
			oppGoal = "l";
		}
		if (side.equals("l"))
		{
			oppGoal = "r";
		}		
	}

/**
 * 	Returns the side of our team
 */
 	
	public String getPlaySide()
	{
		return playOnSide;
	}

/**
 * 	Returns players number
 */
	
	public void setNumber(int num)
	{
		ownNumber = num;
	}
/**
 * 	Sets Playmode
 */

	public void setPlayMode(int mode)
	{
		status = mode;
//		System.out.println("playmode set");
	}
	
	
/**
 *	converts boolean from check HomeZone() to int for playstatus
 */

	public int checkHZone() 
	{
		if (world.myActArea.checkHomeZone())
		{
			return 0;
		}
		else
		{
			return 1;
		}		
	}
/**
 * 	Lets the Player turn to the ball
 */

	public void turnToBall()
	{
		if (!(world.verifyBall())) // already found the ball?
		{
			// no, look around
			actor.turn(40);
		}
	
		if (Math.abs(world.theBall.direction()) > 1.0 && world.theBall.time()!=time) 
		{
			actor.turn(world.theBall.direction());
			// remember the time of the info the action was based on
			// System.out.println("Turning towards ball");
			time=world.theBall.time();
		}
	
	} // turnToBall()
	
	public void setOffside(boolean offside)
	{
		this.offside = offside;
	}

/**
 * 	Updates graphic Information
 */
 
	public void updateGraphic(Vector GraphicList)
	{
		team.GraphicList = GraphicList;
		team.worldModel.m_bildanzeige.repaint();
//		if ( team.newGraphic)
//		{
//			team.worldModel.m_bildanzeige.repaint();
//			team.newGraphic = false;
//		}
//		team.newGraphic = true;
	}

/**
 * 	Returns true if <code>Player</code> is active
 */
 
	public void activate()
	{
		active = true;
	}

/**
 * 	Returns false if <code>Player</code> is inactive
 */
 	
	public void deactivate()
	{
		active = false;
	}
}
